
# q-expansion of newform 136.4.k.b, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 136
weight = 4
poly_data = [2704, 0, 1699337, 0, 4382607, 0, 1120191, 0, 112122, 0, 5319, 0, 119, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-1462819625392, 0, -5240890807749, 0, -776715718098, 0, -35042935761, 0, -553258413, 0, -2958786, 0, -16553, 0], 53247141288], [[-1244553232, -29043895248, -52342238181, 0, -12509813490, 0, -708095937, 0, -2543421, 0, 574494, 0, 9031, 0], 29043895248], [[-1244553232, 29043895248, -52342238181, 0, -12509813490, 0, -708095937, 0, -2543421, 0, 574494, 0, 9031, 0], 29043895248], [[480468824824, 0, 81386133429, 0, 12509813490, 0, 708095937, 0, 2543421, 0, -574494, 0, -9031, 0], 14521947624], [[0, -9487413190220, 0, -26060390244033, 0, -6693378067758, 0, -670840961865, 0, -31833327273, 0, -712145454, 0, -5983429], 377570638224], [[-2125260748364, -51529530090165, -44078757852480, -76569890579577, -14410751366271, -19003094129961, -1686473706255, -1893984144588, -89819078388, -89620518609, -2196992499, -1997695269, -19735339, -16718259], 1038319255116], [[2125260748364, -51529530090165, 44078757852480, -76569890579577, 14410751366271, -19003094129961, 1686473706255, -1893984144588, 89819078388, -89620518609, 2196992499, -1997695269, 19735339, -16718259], 1038319255116], [[21859444421360, 553556528507924, -100278228438336, 1681302766451415, -15301683043296, 436113015710646, -163613325720, 43968354527715, 56784667056, 2094008337087, 2586850656, 46939392582, 30952792, 394782247], 4153277020464], [[-21859444421360, 553556528507924, 100278228438336, 1681302766451415, 15301683043296, 436113015710646, 163613325720, 43968354527715, -56784667056, 2094008337087, -2586850656, 46939392582, -30952792, 394782247], 4153277020464], [[0, -1424787091162888, 0, -3581518823081679, 0, -903371091656622, 0, -89390253958059, 0, -4199517040215, 0, -93193189950, 0, -778104491], 2076638510232], [[0, -313807418169676, 0, -858631979860383, 0, -220556221085274, 0, -22119341247183, 0, -1050433671063, 0, -23515736826, 0, -197687963], 377570638224], [[-10332452117624, -257557705863182, -48633301863731, -633026078177693, -12386311463292, -161046078537220, -1235688069941, -16088764865267, -58709091363, -762476943733, -1318184036, -17048541404, -11120005, -143212757], 230737612248], [[-10332452117624, 257557705863182, -48633301863731, 633026078177693, -12386311463292, 161046078537220, -1235688069941, 16088764865267, -58709091363, 762476943733, -1318184036, 17048541404, -11120005, 143212757], 230737612248]]

hecke_ring_character_values = [[103, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [69, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [105, [0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, 0, 0, 0], [-6, 0, -1, 0, 0, 6, 1, 0, 0, 0, 0, 0, -1, 0], [-8, -1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [9, 0, 1, -1, -1, 4, -1, -1, 0, -2, 1, -1, 0, 1], [0, 0, -2, 2, 0, -3, -1, -1, -1, -1, 1, 1, -1, 1], [-14, 1, -4, 0, 3, 14, 1, 0, 1, 0, 1, -3, -1, 0], [7, 0, 0, -2, -1, 7, 0, -5, 0, -4, 0, -1, 0, 0], [-32, -1, 0, -10, 1, -32, 0, -3, 0, -1, 1, 1, 0, -3], [39, 1, 0, -16, -5, 39, 0, 7, 0, 4, -1, -5, 0, 2], [50, 1, -16, 0, -1, -50, 18, 0, -4, 0, 1, 1, 2, 0], [0, 0, 19, -19, 0, 23, 5, 5, 4, 4, -1, 5, -1, 1], [-21, 5, 18, 18, 5, 0, 3, -3, 0, 0, 0, 0, -3, -3], [0, 0, -24, 24, 0, -79, -20, -20, 4, 4, -3, 9, 1, -1], [0, 0, -33, 33, 0, -17, 12, 12, -6, -6, 3, 5, 4, -4], [-87, -1, -4, 0, 1, 87, 7, 0, 12, 0, -1, -1, 4, 0], [-3, 3, 17, 17, -3, 0, 10, -10, 13, -13, 0, 0, 4, 4], [-241, -2, 0, -34, 6, -241, 0, 20, 0, 5, 2, 6, 0, -8], [-90, 1, 0, 48, -3, -90, 0, -20, 0, 4, -1, -3, 0, -2], [89, 6, -36, 0, -4, -89, 33, 0, 5, 0, 6, 4, -5, 0], [0, 0, -13, 13, 0, -33, 10, 10, 10, 10, -1, -11, -2, 2], [-40, 9, -67, -67, 14, 0, 11, -11, -2, 2, 0, 0, -12, -12], [99, -5, 0, -50, -10, 99, 0, -20, 0, -16, 5, -10, 0, -6], [-124, 3, -7, -7, 28, 0, 4, -4, -24, 24, 0, 0, -4, -4], [344, -4, 60, 60, 16, 0, -5, 5, -2, 2, 0, 0, 5, 5], [-135, -3, 0, 59, -5, -135, 0, -46, 0, -16, 3, -5, 0, 14], [141, 3, -88, 0, 15, -141, 49, 0, -8, 0, 3, -15, -12, 0], [125, 7, 150, 0, 8, -125, -36, 0, -36, 0, 7, -8, -2, 0], [0, 0, -108, 108, 0, 968, 33, 33, 1, 1, -6, -10, -5, 5], [449, -7, 0, 193, -15, 449, 0, -8, 0, 10, 7, -15, 0, 8], [759, 0, 229, 229, 21, 0, -8, 8, 4, -4, 0, 0, -14, -14], [-690, 8, 0, -65, -6, -690, 0, -11, 0, -54, -8, -6, 0, 17], [-330, 28, -24, -24, 12, 0, -6, 6, -12, 12, 0, 0, -28, -28], [0, 0, -8, 8, 0, -586, -78, -78, 47, 47, -12, 16, 6, -6], [599, -13, -18, -18, -13, 0, 65, -65, -12, 12, 0, 0, 21, 21], [-501, -15, 141, 0, 5, 501, -53, 0, 42, 0, -15, -5, 45, 0], [-228, -1, 0, 36, 43, -228, 0, -116, 0, 27, 1, 43, 0, -20], [-763, 10, 0, -62, -3, -763, 0, -75, 0, 36, -10, -3, 0, -38], [0, 0, 180, -180, 0, 405, -22, -22, -13, -13, 9, 29, 12, -12], [220, -4, 176, 0, -14, -220, -3, 0, 48, 0, -4, 14, -22, 0], [609, 39, 70, 70, 71, 0, 26, -26, -22, 22, 0, 0, -26, -26], [-673, -32, 240, 0, 22, 673, 8, 0, 60, 0, -32, -22, 4, 0], [-1406, 5, -34, 0, -10, 1406, 69, 0, -16, 0, 5, 10, 38, 0], [585, 8, 0, 312, 26, 585, 0, 79, 0, 41, -8, 26, 0, -33], [697, -15, 7, 0, -63, -697, -122, 0, 22, 0, -15, 63, 2, 0], [0, 0, 130, -130, 0, -463, 69, 69, 31, 31, 27, 109, 7, -7], [749, -17, 67, 0, 17, -749, -121, 0, 64, 0, -17, -17, -3, 0], [0, 0, 57, -57, 0, 976, -39, -39, -120, -120, 27, 16, 14, -14], [525, -3, 0, -262, 50, 525, 0, -22, 0, 56, 3, 50, 0, -2], [1427, -15, -330, -330, 45, 0, -72, 72, -62, 62, 0, 0, -40, -40], [-827, -12, 0, 88, -42, -827, 0, -62, 0, -136, 12, -42, 0, 24], [779, 5, -77, -77, 83, 0, 29, -29, -91, 91, 0, 0, 1, 1], [0, 0, -139, 139, 0, -2051, 170, 170, -32, -32, -18, -11, -34, 34], [0, 0, 78, -78, 0, 3376, -200, -200, -35, -35, 42, -14, 20, -20], [-832, 8, 0, -132, 4, -832, 0, -131, 0, 60, -8, 4, 0, -2], [-1595, 35, 38, 38, -45, 0, 48, -48, -66, 66, 0, 0, -16, -16], [-1843, -30, 0, -242, -55, -1843, 0, -53, 0, 64, 30, -55, 0, 48], [0, 0, -380, 380, 0, 1001, 45, 45, -102, -102, 39, 37, 0, 0], [774, 4, -317, 0, 32, -774, -31, 0, 140, 0, 4, -32, -17, 0], [342, -6, 246, 246, -16, 0, 6, -6, -44, 44, 0, 0, -76, -76], [3153, 23, -528, -528, 39, 0, 86, -86, 32, -32, 0, 0, 38, 38], [-367, -14, 0, -352, -34, -367, 0, 294, 0, 75, 14, -34, 0, 94], [2167, 70, -228, 0, 100, -2167, 118, 0, -24, 0, 70, -100, -76, 0], [578, -59, 238, 0, -2, -578, -155, 0, 20, 0, -59, 2, 78, 0], [0, 0, 69, -69, 0, 3107, 47, 47, -66, -66, 7, 109, 29, -29], [-799, -14, 0, 848, -6, -799, 0, 30, 0, -244, 14, -6, 0, 132], [307, -47, 319, 0, -9, -307, 511, 0, -64, 0, -47, 9, 69, 0], [0, 0, -336, 336, 0, 2491, -26, -26, 136, 136, 35, 7, 19, -19], [-505, 15, -270, -270, -115, 0, 61, -61, -78, 78, 0, 0, 6, 6], [0, 0, 96, -96, 0, 2917, -73, -73, 41, 41, -21, -35, -51, 51], [86, -67, -150, 0, -179, -86, 8, 0, 249, 0, -67, 179, 96, 0], [1020, 51, -87, -87, 144, 0, 180, -180, -132, 132, 0, 0, -98, -98], [-2844, 20, 0, -341, -16, -2844, 0, -418, 0, 52, -20, -16, 0, -114], [0, 0, -192, 192, 0, 2096, 372, 372, -13, -13, -38, -34, 0, 0], [0, 0, 381, -381, 0, -3326, 1, 1, 124, 124, 65, 130, 72, -72], [-1337, -15, -8, 0, -129, 1337, 209, 0, -348, 0, -15, 129, 2, 0], [-4527, 16, -624, 0, -42, 4527, -174, 0, 48, 0, 16, 42, 80, 0], [5892, 38, 16, 16, 148, 0, -86, 86, -218, 218, 0, 0, -53, -53], [1711, 25, 113, 0, -23, -1711, -438, 0, -180, 0, 25, 23, -34, 0], [-3802, 25, -639, -639, 102, 0, 20, -20, 116, -116, 0, 0, -46, -46], [3952, -85, 624, 0, 77, -3952, 119, 0, 77, 0, -85, -77, 77, 0], [0, 0, 561, -561, 0, 1332, 287, 287, -18, -18, 91, -4, -42, 42], [889, 40, 0, 14, 76, 889, 0, 238, 0, -199, -40, 76, 0, -142], [-105, -51, 67, 67, -25, 0, -300, 300, -9, 9, 0, 0, -42, -42], [3168, -74, -322, 0, 45, -3168, -16, 0, 320, 0, -74, -45, 142, 0], [0, 0, -175, 175, 0, -590, -57, -57, 30, 30, 25, 58, -36, 36], [0, 0, -512, 512, 0, -5861, 450, 450, 4, 4, 19, -109, -59, 59], [3344, 4, 158, 158, -44, 0, -91, 91, -68, 68, 0, 0, 119, 119], [0, 0, -16, 16, 0, 4047, -52, -52, 7, 7, 23, 35, -38, 38], [-5042, -101, 0, 448, -155, -5042, 0, 263, 0, -83, 101, -155, 0, 107], [874, -87, 646, 0, -31, -874, 410, 0, 197, 0, -87, 31, 50, 0], [0, 0, 594, -594, 0, -1617, 401, 401, 27, 27, -69, -161, -15, 15], [1498, 104, 517, 0, 194, -1498, -358, 0, -396, 0, 104, -194, -214, 0], [936, -9, -670, 0, 79, -936, 158, 0, -231, 0, -9, -79, 46, 0], [6448, -44, 250, 250, -142, 0, 106, -106, -168, 168, 0, 0, 110, 110], [-1088, -131, 108, 0, -25, 1088, -474, 0, -248, 0, -131, 25, -36, 0], [969, 111, -127, -127, 273, 0, -17, 17, 55, -55, 0, 0, -169, -169], [-955, 35, 0, -2060, 13, -955, 0, 53, 0, 40, -35, 13, 0, 164], [-5477, 3, 0, 29, -109, -5477, 0, -378, 0, -126, -3, -109, 0, 14], [-1220, -169, -391, -391, -396, 0, 318, -318, 280, -280, 0, 0, 102, 102], [0, 0, -769, 769, 0, 3427, 391, 391, -224, -224, 59, 185, 101, -101], [0, 0, 166, -166, 0, -2175, -417, -417, 384, 384, -47, 3, 107, -107], [4331, 113, -63, 0, 189, -4331, -24, 0, -188, 0, 113, -189, 56, 0], [9953, -118, 605, 605, -257, 0, -18, 18, 40, -40, 0, 0, 130, 130], [0, 0, 508, -508, 0, 295, 72, 72, 203, 203, 11, -197, 30, -30], [0, 0, -846, 846, 0, 4235, 101, 101, 8, 8, -85, -19, -189, 189], [-1074, -110, -270, -270, 14, 0, 237, -237, 180, -180, 0, 0, -129, -129], [-5510, 165, -868, 0, 197, 5510, -62, 0, 324, 0, 165, -197, 120, 0], [8, 63, 0, -534, 29, 8, 0, 519, 0, 143, -63, 29, 0, -73], [3226, -56, 424, 424, 88, 0, 94, -94, -164, 164, 0, 0, 24, 24], [1443, 65, 0, 838, -298, 1443, 0, 938, 0, -324, -65, -298, 0, 40], [-5695, 3, -915, 0, 39, 5695, -1081, 0, -82, 0, 3, -39, 65, 0], [0, 0, -1326, 1326, 0, 9651, 530, 530, 153, 153, -95, 43, 78, -78], [-385, 109, 0, 2058, -72, -385, 0, -268, 0, -268, -109, -72, 0, 192], [-164, -140, 0, -851, 114, -164, 0, -492, 0, -78, 140, 114, 0, -272], [-2547, 11, 1462, 1462, -141, 0, -506, 506, 162, -162, 0, 0, 210, 210], [3014, 81, -650, 0, 56, -3014, 9, 0, 128, 0, 81, -56, 4, 0], [12819, 81, -218, -218, -7, 0, 199, -199, -132, 132, 0, 0, 221, 221], [0, 0, 1010, -1010, 0, 1808, 115, 115, -328, -328, 176, -74, 36, -36], [2556, -8, -2102, 0, -149, -2556, -76, 0, 140, 0, -8, 149, 172, 0], [-2716, 39, 0, -3254, -64, -2716, 0, 25, 0, 500, -39, -64, 0, 8], [-888, 240, 0, 253, 6, -888, 0, -150, 0, 282, -240, 6, 0, 70], [3574, 64, 1859, 0, -32, -3574, -211, 0, 352, 0, 64, 32, 19, 0], [650, -231, 1809, 1809, -426, 0, 526, -526, 120, -120, 0, 0, 266, 266], [-3781, 176, 0, 1818, 279, -3781, 0, 57, 0, 464, -176, 279, 0, -458], [6649, -148, 0, 506, -148, 6649, 0, 404, 0, 429, 148, -148, 0, -292], [-633, 249, -938, -938, 25, 0, 169, -169, 192, -192, 0, 0, 23, 23], [0, 0, 42, -42, 0, -1730, -675, -675, -60, -60, 18, -464, -34, 34], [0, 0, -32, 32, 0, -4717, 844, 844, -29, -29, 175, -425, -274, 274], [-3690, 53, 0, -826, -329, -3690, 0, -765, 0, 23, -53, -329, 0, 143], [-556, -239, 0, 1470, -135, -556, 0, 306, 0, 21, 239, -135, 0, 34], [0, 0, 2153, -2153, 0, 3988, 111, 111, 396, 396, -69, -352, 18, -18], [-9913, 116, 697, 697, 673, 0, 316, -316, -72, 72, 0, 0, -324, -324], [-11291, 42, -1667, -1667, -157, 0, 60, -60, -128, 128, 0, 0, -168, -168], [0, 0, -1661, 1661, 0, -4132, -579, -579, 108, 108, 121, -220, -300, 300], [930, -184, 0, -435, -324, 930, 0, -357, 0, -1196, 184, -324, 0, 267], [0, 0, 1518, -1518, 0, 6132, -265, -265, 220, 220, 80, -278, 52, -52], [739, 123, -1862, 0, -22, -739, 444, 0, -536, 0, 123, 22, 98, 0], [6353, -117, 0, -693, -173, 6353, 0, 577, 0, -206, 117, -173, 0, 297], [30, -211, 0, -1470, 138, 30, 0, 293, 0, -188, 211, 138, 0, -384], [-371, -2, -2284, 0, -228, 371, -1139, 0, -175, 0, -2, 228, -77, 0], [-2964, -162, -277, 0, -90, 2964, 396, 0, 466, 0, -162, 90, -220, 0], [229, 49, -1270, -1270, -383, 0, -559, 559, -208, 208, 0, 0, 319, 319], [3730, 43, 2020, 0, 151, -3730, 704, 0, 251, 0, 43, -151, -568, 0], [12765, -17, 0, 1780, 399, 12765, 0, 193, 0, -160, 17, 399, 0, -130], [-2993, 68, -1856, 0, 170, 2993, 922, 0, -92, 0, 68, -170, -282, 0], [0, 0, -113, 113, 0, 9097, 50, 50, -726, -726, 233, -845, -338, 338], [-7318, 90, -1338, -1338, 18, 0, 571, -571, -544, 544, 0, 0, -511, -511], [10333, -1, 1992, 0, -29, -10333, 433, 0, -312, 0, -1, 29, -64, 0], [-3902, 7, 0, 720, -85, -3902, 0, 72, 0, -128, -7, -85, 0, -240], [1841, -341, 37, 37, -831, 0, -17, 17, -121, 121, 0, 0, 203, 203], [3856, 17, 0, 1552, 303, 3856, 0, 633, 0, 397, -17, 303, 0, -215], [-2353, -93, 1731, 0, 75, 2353, 411, 0, -840, 0, -93, -75, 253, 0], [9309, -44, 2000, 0, 504, -9309, -506, 0, 495, 0, -44, -504, 126, 0], [-12071, 91, -1942, -1942, -89, 0, -37, 37, -44, 44, 0, 0, -203, -203], [-3290, 265, 1576, 0, 135, 3290, 216, 0, -124, 0, 265, -135, 48, 0], [0, 0, -218, 218, 0, -8046, -1166, -1166, -262, -262, 88, 1028, 95, -95], [-3325, 84, -890, 0, 29, 3325, -131, 0, -540, 0, 84, -29, 50, 0], [-5436, -18, 0, 1565, 586, -5436, 0, -913, 0, -516, 18, 586, 0, -73], [-10320, -375, 1065, 1065, -238, 0, -985, 985, 318, -318, 0, 0, 206, 206], [0, 0, 1098, -1098, 0, 11927, -394, -394, -75, -75, -199, 327, -118, 118], [0, 0, -587, 587, 0, 2393, -871, -871, 630, 630, -39, 799, 443, -443], [0, 0, 2014, -2014, 0, -13141, -311, -311, -176, -176, -5, -71, 41, -41], [2343, 10, 0, 4318, 174, 2343, 0, 542, 0, 441, -10, 174, 0, -326], [2387, 118, 0, 98, 802, 2387, 0, 754, 0, -1103, -118, 802, 0, 50], [5979, -48, -406, 0, -555, -5979, 1543, 0, 32, 0, -48, 555, 332, 0]]
